import { FLORIS_PLOTS, getWindmillData } from '@api/floris';
import { getParks, Park } from '@api/wind';
import {
  Autocomplete,
  Button,
  Checkbox,
  CheckboxGroup,
  DateInput,
  Heading,
} from '@components/ui';
import { Controller, useForm } from '@utils/form';
import clsx from 'clsx';
import React, { useEffect, useState } from 'react';

import { FlorisTable } from '../floris-table/component';
import styles from './styles.module.scss';
import { FlorisFormProps, FlorisFormValues } from './types';

export function FlorisForm({
  onSuccess,
  onFail,
  className,
  ...props
}: FlorisFormProps) {
  const [pending, setPending] = useState<boolean>(false);
  const [parks, setParks] = useState<Park[]>([]);
  const [isManualEntry, setIsManualEntry] = useState<boolean>(false);
  const { control, reset, getValues } = useForm<FlorisFormValues>({});

  const fetchParks = async () => {
    const res = await getParks();
    setParks(res.data);
  };

  useEffect(() => {
    fetchParks();
  }, []);

  const validate = (values: Partial<FlorisFormValues>) => {
    console.log(values);
    return true;
  };

  const handleSubmit = async (event: React.FormEvent) => {
    event.preventDefault();
    const values = getValues();
    if (!validate(values)) {
      return;
    }
    setPending(true);
    const { data, error } = await getWindmillData(values);
    if (data) {
      onSuccess(data, values.dateFrom);
    } else {
      onFail(error.message);
    }
    setPending(false);
  };

  const handleResetButtonClick = () => {
    reset({});
  };

  const handleManulEntryCheckboxChange = (
    event: React.ChangeEvent<HTMLInputElement>,
  ) => {
    const { checked } = event.target;
    setIsManualEntry(checked);
    if (checked) {
      reset({ ...getValues(), park: undefined });
    } else {
      reset({ ...getValues(), turbines: undefined });
    }
  };

  return (
    <form
      onSubmit={handleSubmit}
      className={clsx(className, styles.form)}
      {...props}
    >
      <Heading tag="h3">Turbines properties</Heading>
      <div className={styles.content}>
        <div className={styles.part}>
          <div className={styles.dateRangeBox}>
            <Controller
              {...control('dateFrom')}
              render={(params) => <DateInput placeholder="from" {...params} />}
            />
            <Controller
              {...control('dateTo')}
              render={(params) => <DateInput placeholder="to" {...params} />}
            />
          </div>
          <Checkbox
            label={{ text: 'Manual entry', position: 'right' }}
            onChange={handleManulEntryCheckboxChange}
          />
          {isManualEntry && (
            <Controller
              {...control('turbines')}
              render={(params) => <FlorisTable {...params} />}
            />
          )}
          {!isManualEntry && (
            <Controller
              {...control('park')}
              render={(params) => (
                <Autocomplete
                  options={parks}
                  getOptionKey={(p) => p.id}
                  getOptionLabel={(p) => p.name}
                  {...params}
                />
              )}
            />
          )}
        </div>
        <div>
          <Controller
            {...control('plots')}
            render={(params) => (
              <CheckboxGroup
                items={Object.values(FLORIS_PLOTS)}
                getItemKey={(i) => i.name}
                getItemLabel={(i) => i.label}
                label="Plots"
                {...params}
              />
            )}
          />
        </div>
      </div>
      <div className={styles.buttonBox}>
        <Button type="submit" pending={pending}>
          Submit
        </Button>
        <Button variant="secondary" onClick={handleResetButtonClick}>
          Reset
        </Button>
      </div>
    </form>
  );
}
